#!/usr/bin/env python3
import torchaudio as ta
from chatterbox.tts import ChatterboxTTS
import json
from os import makedirs
from os.path import join
from os.path import exists
import re
from os import remove
import whisper
from piper_phonemize import phonemize_espeak

chatterbox_model = ChatterboxTTS.from_pretrained(device="cpu")

csv_fp = "LJSpeech-1.1/metadata.csv"
wav_dir = "LJSpeech-1.1/wavs"

csv = []

makedirs(wav_dir, exist_ok=True)

whisper_model = whisper.load_model("turbo")


def tts(text, fp, overwrite=False):
    if exists(fp) and not overwrite:
        return

    # low exaggeration so it sounds more neutral
    wav = chatterbox_model.generate(
        text, audio_prompt_path="nina2.wav", exaggeration=0.2
    )
    ta.save(fp, wav, chatterbox_model.sr)


def normalise_text(text, gaps=True):
    return re.sub(r"\W+", " " if gaps else "", text).strip()


def verify(fp, text):
    result = whisper_model.transcribe(fp)["text"].strip()

    verified = phonemize_espeak(normalise_text(text), "en-us") == phonemize_espeak(
        normalise_text(result), "en-us"
    ) or phonemize_espeak(
        normalise_text(text, gaps=False), "en-us"
    ) == phonemize_espeak(
        normalise_text(result, gaps=False), "en-us"
    )

    if not verified:
        print(f"Verification failed for {fp}: expected '{text}', got '{result}'")

    return verified


with open("corpus.txt", "r") as f:
    lines = f.read().splitlines()

for id, text in enumerate(lines):
    print(id + 1, "/", len(lines), text)
    fp = join(wav_dir, f"{id}.wav")

    for x in range(3):
        tts(text, fp, overwrite=x > 0)

        if verify(fp, text):
            break

        print(f"Verification failed for {id}.wav, trying again...")
    else:
        print(f"Verification failed for {id}.wav after {x} attempts, skipping...")
        remove(fp)
        continue

    csv.append((id, text))

with open(csv_fp, "w") as f:
    for id, text in csv:
        # Write the CSV file in LJSpeech format
        f.write(f"{id}|{text}\n")


print(f"Score: {len(csv)}/{len(lines)} ({len(csv) / len(lines) * 100:.0f}%)")
